const appName = '{{ project }}'
const appCacheName = '{{ project }}-{{ version }}';

const preCacheFiles = [{{ pre_cache }}];

const cachePatterns = [{{ cache_patterns }}];

self.addEventListener('install', (e) => {
  console.log('[Service Worker] Install');
  e.waitUntil((async () => {
    const cache = await caches.open(appCacheName);
    console.log('[Service Worker] Caching ');
    await cache.addAll(preCacheFiles);
  })());
});

self.addEventListener('activate', (event) => {
  console.log('[Service Worker] Activating');
  event.waitUntil((async() => {
    const cacheNames = await caches.keys();
    for (const cacheName of cacheNames) {
      if (cacheName.startsWith(appName) && cacheName !== appCacheName) {
	console.log(`[Service Worker] Delete old cache ${cacheName}`);
        caches.delete(cacheName);
      }
    }
  })());
  return self.clients.claim();
});

self.addEventListener('fetch', (e) => {
  let enableCache = e.request.url.startsWith(self.location.origin);
  for (const pattern of cachePatterns) {
    if (e.request.url.startsWith(pattern)) {
      enableCache = true
      break
    }
  }
  if (!enableCache) {
    return
  }
  e.respondWith((async () => {
    const cache = await caches.open(appCacheName);
    let response = await cache.match(e.request);
    console.log(`[Service Worker] Fetching resource: ${e.request.url}`);
    if (response) { return response; }
    response = await fetch(e.request);
    console.log(`[Service Worker] Caching new resource: ${e.request.url}`);
    cache.put(e.request, response.clone());
    return response;
  })());
});
